<?php
// $Id: ec_charge.admin.inc,v 1.1.2.10 2008/01/23 23:15:11 sime Exp $

/**
 * @author e-Commerce Dev Team
 * @file ec_charge.admin.inc
 * All admin function is located here to avoid useless module footprint
 */

define('EC_CHARGE_CHARGE', 1);
define('EC_CHARGE_CHARGE_PCT_SUBTOTAL', 2);
define('EC_CHARGE_CHARGE_PCT_ITEMTOTAL', 3);


/**
 * List all charges and new charge options
 */
function _ec_charge_overview_page() {
  // First we display a list of currently configured charges.

  $output = drupal_get_form('_ec_charge_add');
  drupal_set_title('Charge configuration');

  if ($charges = ec_charge_get_charges()) {
    $operators = _ec_charge_operators('list');
    $charge_list = array();
    foreach ($charges AS $charge) {
      $row['name']   = l($charge['display'], 'admin/ecsettings/charges/edit/'. $charge['chid']);
      $row['weight'] = $charge['position'];
      $row['operations'] = l('Settings', 'admin/ecsettings/charges/edit/'. $charge['chid'] .'/settings').',&nbsp&nbsp '. l('Filters', 'admin/ecsettings/charges/edit/'. $charge['chid'] .'/filters') .',&nbsp&nbsp '. l('Delete', 'admin/ecsettings/charges/delete/'. $charge['chid']);
      array_push($charge_list, $row);
    }
    $output .= theme('fieldset', array(
      '#title'       => t('Currently configured charges'),
      '#children'    => theme('table', array('Name', 'Weight', 'Operations'), $charge_list),
      '#collapsible' => true,
      '#collapsed'   => false)
    );
  }
  return $output;
}

function _ec_charge_add_form() {
  $form['box'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Add a new charge'),
    '#collapsible'    => true,
    '#collapsed'      => true,
  );
  $form['box']['name'] = array(
    '#type'           => 'textfield',
    '#title'          => 'Charge name',
    '#description'    => 'This is the description for the review screen and invoice.',
  );
  $form['box']['submit'] = array(
    '#type'          => 'submit',
    '#value'         => t('Create charge'),
  );
  return $form;
}

function _ec_charge_add_submit($form_id, $form_values) {
  db_query("INSERT INTO {ec_charge} (display) VALUES ('%s')", $form_values['name']);
  drupal_set_message(t("New charge '%name' has been created.", array('%name' => $form_values['name']))); 
  $chid = db_result(db_query('SELECT max(chid) FROM {ec_charge}'));
  return 'admin/ecsettings/charges/edit/'. $chid .'/settings';
}

function _ec_charge_edit_settings_form() {
  $chid = arg(4);
  $operators[0] = '--';
  $operators = _ec_charge_operators('list');
  $charge = ec_charge_get_charges($chid);
  
  $breadcrumbs = drupal_get_breadcrumb();
  $breadcrumbs[] = $charge['display'];
  drupal_set_breadcrumb($breadcrumbs);  
  
  drupal_set_title(t("Edit the '%name' charge", array('%name' => t($charge['display']))));

  $form['cart'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Checkout'),
    '#description'    => t('Determine how your charge will interact with the checkout review screen.'),
    '#collapsible'    => true,
    '#collapsed'      => false,
  );
  $form['cart']['display'] = array(
    '#type'           => 'textfield',
    '#title'          => 'Charge name',
    '#required'       => true,
    '#default_value'  => $charge['display'],
    '#description'    => 'This is the description for the review screen and invoice.',
  );
  $form['cart']['position'] = array(
    '#type'          => 'textfield',
    '#required'      => true,
    '#title'         => t('Calculation order'),
    '#description'   => t('Position of this charge in the ec_charge calculations.'),
    '#default_value' => $charge['position'],
    '#size'          => 5,
  );
  $form['cart']['subtotal_before'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal before?'),
    '#default_value' => $charge['subtotal_before'],
  );
  $form['cart']['subtotal_after'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal after?'),
    '#default_value' => $charge['subtotal_after'],
  );
  $form['cart']['already_added'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display this amount as \'included\', but don\'t add it to the subtotal.'),
    '#default_value' => $charge['already_added'],
  );
  $form['cart']['hide_if_zero'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Suppress this row in the cart if the total charge = 0.'),
    '#default_value' => $charge['hide_if_zero'],
  );
  $form['chid'] = array(
    '#type'          => 'hidden',
    '#value'         => $chid,
  );
  $form['filters'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Filters'),
    '#description'    => t('Filters are going to go here.'),
    '#collapsible'    => true,
    '#collapsed'      => true,
  );
  $use_auto = ($chid) ? $charge['use_auto'] : true;
  $form['auto'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Calculation') . (($use_auto) ? '' : ' (Disabled)'),
    '#description'    => t('Set these values for automated charge calculation.'),
    '#collapsible'    => true,
    '#collapsed'      => !$use_auto,
  );
  $form['auto']['use_auto'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable'),
    '#default_value' => $user_auto,
    '#options' => $operators,
    );
  $form['auto']['operator'] = array(
    '#type' => 'select',
    '#title' => t('Operator'),
    '#default_value' => ($chid) ? $charge['operator'] : EC_CHARGE_ADD,
    '#options' => $operators,
    );
  $form['auto']['rate'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Charge rate'),
    '#default_value' => $charge['rate'],
    '#description'   => t('Use positive or negative values. You can specify percent like 10 or -20, use the operator to specify percentage.'),
    '#size'          => 8,
  );
  $use_php = ($chid) ? $charge['use_custom'] : true;
  $form['custom'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('PHP Calculation') . (($use_auto) ? '' : ' (Disabled)'),
    '#description'    => t('Define php for custom calculation. Enclose in <?php tags ?>.'),
    '#collapsible'    => true,
    '#collapsed'      => !$use_php,
  );
  $form['custom']['use_custom'] = array(
    '#type' => 'select',
    '#title' => t('Enable'),
    '#default_value' => $user_custom,
    '#options' => $operators,
    );

  $form['submit'] = array(
    '#type'          => 'submit',
    '#value'         => t('Save charge'),
  );

//  return _ec_charge_admin_form();
  return $form;
}

function _ec_charge_edit_settings_submit($form_id, $form_values) {
  db_query("UPDATE {ec_charge} SET display = '%s', operator = %d, rate = %f,
    position = %d, subtotal_before = %d, subtotal_after = %d, already_added = %d,
    hide_if_zero = %d WHERE chid = %d",
    $form_values['display'], $form_values['operator'],
    $form_values['rate'], $form_values['position'],
    $form_values['subtotal_before'], $form_values['subtotal_after'],
    $form_values['already_added'], $form_values['hide_if_zero'],
    $form_values['chid']);
  return 'admin/ecsettings/charges';
}

function _ec_charge_delete_form() {
  $chid = arg(4);

  $charges = ec_charge_get_charges($chid);
  $form['chid'] = array('#type' => 'hidden', '#default_value' => $chid);
  return confirm_form(
      $form,
      t("Are you sure you want to delete the charge '%name'?",
        array('%name' => $charges['display'])),
      'admin/ecsettings/charges',
      t('This action can not be undone, you will lose all information associated with this charge.'),
      t('Delete'), t('Cancel'));
}

function  _ec_charge_delete_submit($form_id, $form_values) {
  db_query('DELETE FROM {ec_charge} WHERE chid = %d', $form_values['chid']);
  drupal_set_message(t('The charge has been deleted.'));
  return 'admin/ecsettings/charges';
}












/**
 * Creating/editing charge form
 */
function _ec_charge_admin_form() {
  $charge_methods = _ec_charge_charge_methods();

  switch (arg(3)) {
    case 'add':
      drupal_set_title(t('New charge'));
      $op = 'add';
      $chid = 0;
      $provider = $_GET['provider'];
      $method = $_GET['method'];
      break;
    case 'edit':
      $op = 'edit';
      $chid = arg(4);
      include_once drupal_get_path('module', 'ec_charge') .'/ec_charge.inc';
      $charge = _ec_charge_get_charges($chid);
      $provider = $charge->provider; // convenience
      $method = $charge->method; // convenience
      drupal_set_title(t('Edit') .' '. check_plain($charge->display));
      break;
  }

  $form = array();
  $form['module'] = array(
    '#type'           => 'fieldset',
    '#title'          => 'Flexicharge defaults',
    '#description'    => 'Charge description goes here',
    '#collapsible'    => true,
    '#collapsed'      => false,
    '#tree'           => true
  );
  $form['module']['chid'] = array(
    '#type'           => 'value',
    '#value'          => $chid
  );
  $form['module']['provider'] = array(
    '#type'           => 'value',
    '#value'          => $provider
  );
  $form['module']['method'] = array(
    '#type'           => 'value',
    '#value'          => $method
  );
  $form['module']['display'] = array(
    '#type'           => 'textfield',
    '#required'       => true,
    '#title'          => t('Display label'),
    '#description'    => t('This is the description for the review screen and invoice.'),
    '#default_value'  => ($chid) ? $charge->display : $charge_methods[$provider][$method]['#title'],
    '#size' => 20,
  );
  $operators[0] = '--';
  $operators = _ec_charge_operators('list');

  $form['module']['elements'] = array();

  $form['module']['operator'] = array(
    '#type' => 'select',
    '#title' => t('Operator'),
    '#default_value' => ($chid) ? $charge->operator : EC_CHARGE_ADD,
    '#options' => $operators,
    );
  $form['module']['rate'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Charge rate'),
    '#default_value' => ($chid) ? $charge->rate : 0,
    '#description'   => t('Use positive or negative values. You can specify percent like 10 or -20, use the operator to specify percentage.'),
    '#size'          => 8,
  );
  $form['module']['position'] = array(
    '#type'          => 'textfield',
    '#required'      => true,
    '#title'         => t('Calculation order'),
    '#description'   => t('Position of this charge in the ec_charge calculations.'),
    '#default_value' => ($chid) ? $charge->position : 1,
    '#size'          => 5,
  );
  $form['module']['subtotal_before'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal before?'),
    '#default_value' => ($chid) ? $charge->subtotal_before : 0,
  );
  $form['module']['subtotal_after'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display a subtotal after?'),
    '#default_value' => ($chid) ? $charge->subtotal_after : 0,
  );
  $form['module']['already_added'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Display this amount as \'included\', but don\'t add it to the subtotal.'),
    '#default_value' => ($chid) ? $charge->already_added : 0,
  );
  $form['module']['hide_if_zero'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Suppress this row in the cart if the total charge = 0.'),
    '#default_value' => ($chid) ? $charge->hide_if_zero : 0,
  );
/*
  $ptypes = product_ptypes_get('names');
  $form['module']['ptypes'] = array(
    '#type'          => 'select',
    '#title'         => t('Product types'),
    '#options'       => $ptypes,
    '#multiple'      => true,
    '#default_value' => empty($charge->ptypes) ? 0 : $charge->ptypes,
    '#description'   => 'Please choose the products that this charge applies to. No selection indicates all products. Charge will apply only once, if any product in the cart matches this list.',
  );
*/
  $all_roles = user_roles();
  $form['module']['roles'] = array(
    '#type'          => 'select',
    '#title'         => t('Roles'),
    '#options'       => $all_roles,
    '#multiple'      => true,
    '#default_value' => (($charge->roles) ? $charge->roles : 0),
    '#description'   => 'Please select roles that this charge applies to. No selection indicates all roles. Charge will apply only once, if user is in any selected role.',
  );

  // Get the form elements required by the charge provider
  $f = $provider .'_ec_charge_form_elements';
  if (function_exists($f)) {
    //$form passed by array
    $f($form, $op, $method);
    $form['elements']['#tree'] = true;
  }

  // If there are custom form elements - apply them from the database values
  if (arg(3) == 'edit' && is_array($charge->settings)) {
    foreach ($charge->settings as $e => $s) {
      foreach ($form['elements'] AS $f => $fieldset) {
        if (isset($form['elements'][$f][$e])) {
          $form['elements'][$f][$e]['#default_value'] = $s;
        }
      }
    }
  }
  $form['submit'] = array(
    '#type'          => 'submit',
    '#value'         => t('Save charge'),
  );

  return $form;
}

/**
 * Implementation of hook_submit().
 *
 * Creating/editing charge form
 */
function _ec_charge_admin_form_submit($form_id, $form) {
  // Get the form elements required by the charge provider
  $f = $provider .'_ec_charge_form_validate';
  if (function_exists($f)) {
    $form = $f($form_id, $form);
  }

  $settings = array();
  if (isset($form['elements'])) {
    foreach ($form['elements'] AS $fieldset) {
      $settings = array_merge($fieldset, $settings);
    }
    $settings = serialize($settings);
  }
  else {
    $settings = '';
  }

  $ptypes = serialize($form['module']['ptypes']);
  $roles = serialize($form['module']['roles']);

  if (!empty($form['module']['chid'])) {
    db_query("UPDATE {ec_charge} SET display = '%s', operator = %d, rate = %f,
      position = %d, subtotal_before = %d, subtotal_after = %d, already_added = %d,
      hide_if_zero = %d, ptypes = '%s', roles = '%s', settings = '%s' WHERE chid = %d",
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings, $form['module']['chid']);
  }
  else {
    db_query("INSERT INTO {ec_charge} ( provider, method, display, operator,
      rate, position, subtotal_before, subtotal_after, already_added, hide_if_zero,
      ptypes, roles, settings) VALUES ('%s', '%s', '%s', %d, %f, %d, %d, %d, %d,
      %d, '%s', '%s', '%s')", $form['module']['provider'], $form['module']['method'],
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings);
  }

  drupal_set_message(t('The charge has been saved.'));
  drupal_goto('admin/ecsettings/charges');
}

/**
 * Delete a charge
 */
function _ec_charge_charge_delete() {
  db_query('DELETE FROM {ec_charge} WHERE chid = %d', arg(4));
  drupal_goto('admin/ecsettings/charges');
}

/**
 * This function mimics shipcalc_shipping_methods() and for cross-reference
 * purposes I have kept it mostly the same. In shipcalc, a 'partner' is UPS.
 * USPS etc, in ec_charge, a 'partner' is actually a collection of one or
 * more charge types.
 *
 * @return
 *   Array, ec_charge methods.
 */
function _ec_charge_charge_methods() {
  include_once drupal_get_path('module', 'ec_charge') .'/ec_charge.inc';
  $methods = array();
  $partners = _ec_charge_partners();
  foreach ($partners as $partner) {
    $charge_methods = $partner->name .'_charge_methods';
    if (function_exists($charge_methods)) {
      $methods = array_merge($methods, $charge_methods(NULL));
    }
  }
  return $methods;
}

/**
 * List all possible operators
 */
function _ec_charge_operators($op = 'list') {
  switch ($op) {
    case 'list':
      return array(
        EC_CHARGE_CHARGE                => 'Simple +/-',
        EC_CHARGE_CHARGE_PCT_SUBTOTAL   => '% of Subtotal',
        EC_CHARGE_CHARGE_PCT_ITEMTOTAL  => '% of Item Total',
        );
  }
}
